#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
Usage: lev_distance.py <words_file> <matrix_file> <file_to_write>

Arguments:
    words – word list
    matrix – matrix with the letters
    file to write – file to save the results
Arguments have to be specified in the above-mentioned order separated with space.
Please, specify the .xlsx for files.
"""

import numpy
from openpyxl.workbook import Workbook
from openpyxl import load_workbook
import docopt


def read_excel(file, col1, col2):
    l1_words = []
    l2_words = []
    rb = load_workbook(filename=file, read_only=True, data_only=True)
    rs = rb.worksheets[0]
    row_num = rs.max_row
    for row_index in range(1, row_num + 1):
        l1_words.append(rs[row_index][col1].value)
        l2_words.append(rs[row_index][col2].value)
    l1_words = filter(None, l1_words)
    l2_words = filter(None, l2_words)
    l1_words = [x.replace(u'\xa0', u'').lower().strip() for x in list(l1_words)]
    l2_words = [x.replace(u'\xa0', u'').lower().strip() for x in list(l2_words)]
    return l2_words, l1_words


def read_excel_matrix(filename):
    rb = load_workbook(filename=filename, read_only=True, data_only=True)
    rs = rb.worksheets[0]
    matrix = numpy.zeros((rs.max_row - 1, rs.max_column - 1))
    # -------------------------------
    language_cell = rs.cell(row=1, column=1).value
    language1_letters = {}
    language2_letters = {}
    # -------------------------------
    for index, rownum in enumerate(range(1, rs.max_row)):
        if index == 0:
            tmp = list(rs[rownum])[1:]
            for ind, letter in enumerate(tmp):
                language2_letters[letter.value] = ind
        else:
            row = list(rs[rownum])
            row = [x.value for x in row]
            language1_letters[row[0]] = index - 1
            matrix[index] = row[1:]
    return language_cell, language1_letters, language2_letters, matrix[1:]


def levenshtein(s1, s2, lang1_words, lang2_words, lang_cell, lang1, lang2, letter_matrix):
    flag = False
    if len(s1) < len(s2):
        return levenshtein(s2, s1, lang2_words, lang1_words, lang_cell, lang1, lang2, letter_matrix)
    if len(s2) == 0:
        return len(s1)

    if lang1_words.lower() == lang_cell.split('/')[0].lower() and lang2_words.lower() == lang_cell.split('/')[1].lower():
        pass
    elif lang1_words.lower() == lang_cell.split('/')[1].lower() and lang2_words.lower() == lang_cell.split('/')[0].lower():
        tmp = lang1
        lang1 = lang2
        lang2 = tmp
        flag = True
    else:
        print ("The table provided has languages that don't coincide with word lists languages. Please provide appropriate table")
        return 0, 0, 0, 0, 0
    previous_row = []
    for i in range(len(s2) + 1):
        previous_row.append((i,i,0,0,["D"]*i))
    for i, c1 in enumerate(s1):
        current_row = [(i+1,0,i+1,0, ["I"]*(i+1))]
        for j, c2 in enumerate(s2):
            if flag == False:
                comp = float(letter_matrix[lang1[c1], lang2[c2]])
            else:
                comp = float(letter_matrix[lang2[c2], lang1[c1]])
        # hypothesize insertion
            total, deletions, insertions, substitutions, path = previous_row[j+1]
            insert = (total+1, deletions, insertions+1, substitutions, path+["I%s" % (c1)])
        # hypothesize deletion
            total, deletions, insertions, substitutions, path = current_row[j]
            delete = (total+0.8, deletions+1, insertions, substitutions, path+["D%s" % (c2)])
        # hypothesize substitution
            total, deletions, insertions, substitutions, path = previous_row[j]
            substitute = (total + comp, deletions, insertions, substitutions+comp, path+["S%s%s" % (c1,c2)])
            current_row.append(min(insert, delete, substitute))
        previous_row = current_row
    return previous_row[-1]


def print_line(list_s1,list_s2, filename_to, lang_cell, lang1, lang2, letter_matrix):
    wb = Workbook()
    ws1 = wb.active
    ws1.title = "result sheet"
    #---------------------------
    lang1_words = str(list_s1[0])
    lang2_words = str(list_s2[0])
    flag_lang = False
    #---------------------------
    lev_dist = []
    ws1.append(["wd l1", "wd l2", "levdist", "levdist norm", "length wdl1", "length wd2"])
    for enum, word in enumerate(list_s1[1:]):
        s1 = word
        s2 = list_s2[enum+1]
        if len(s1) < len(s2):
            tmp = s2
            s2 = s1
            s1 = tmp
            tmp_let = lang1_words
            lang1_words = lang2_words
            lang2_words = tmp_let
            flag_lang = True
        T, D, I, S, P = levenshtein(s1, s2, lang1_words, lang2_words, lang_cell, lang1, lang2, letter_matrix)
        if (T, D, I, S, P) != (0, 0, 0, 0, 0):
            cells1 = []
            cells2 = []
            i1 = 0
            i2 = 0
            for p in P:
                if p.startswith("D"):
                    cells1.append("")
                    cells2.append(s2[i2])
                    i2 += 1
                elif p.startswith("I"):
                    cells2.append("")
                    cells1.append(s1[i1])
                    i1 += 1
                elif p.startswith("S"):
                    cells1.append(s1[i1])
                    i1 += 1
                    cells2.append(s2[i2])
                    i2 += 1
            lev_dist.append(T)
            if flag_lang == True:
                tmp_lang = lang1_words
                lang1_words = lang2_words
                lang2_words = tmp_lang
                flag_lang = False
                tmp_wr = [s2, s1, str(float(T)), str(float(T)/float(len(P))), str(len(s2)), str(len(s1))]
                ws1.append(tmp_wr)
            else:
                tmp_wr = [s1, s2, str(float(T)), str(float(T)/float(len(P))), str(len(s1)), str(len(s2))]
                ws1.append(tmp_wr)
    wb.save(filename_to)


if __name__ == '__main__':
    try:
        arguments = docopt.docopt(__doc__)
        words_file = arguments["<words_file>"]
        matrix_file = arguments["<matrix_file>"]
        file_to_write = arguments["<file_to_write>"]
    except docopt.DocoptExit as e:
        print (e.message)
    l1_words, l2_words = read_excel(words_file, 0, 1)
    lang_cell, lang1, lang2, letter_matrix = read_excel_matrix(matrix_file)
    print_line(l2_words, l1_words, file_to_write, lang_cell, lang1, lang2, letter_matrix)

