#!/usr/bin/env python
# -*- coding:utf-8 -*-

"""
Usage: surprisal.py <words_file> <matrix_file> <file_to_write> <lang_from> <lang_to>

Arguments:
    words – word list
    matrix – matrix with the letters
    file to write – file to save the results
    language from – language to start surprisal from, directionwise
    language to – language to end surprisal with, directionwise
Arguments have to be specified in the above-mentioned order separated with space.
Please, specify the .xlsx for files.
"""

from __future__ import division
from math import log
import numpy
from openpyxl.workbook import Workbook
from openpyxl import load_workbook
import sys
import docopt


def read_excel(file, col1, col2):
    l1_words = []
    l2_words = []
    rb = load_workbook(filename=file, read_only=True, data_only=True)
    rs = rb.worksheets[0]
    row_num = rs.max_row
    for row_index in range(1, row_num + 1):
        l1_words.append(rs[row_index][col1].value)
        l2_words.append(rs[row_index][col2].value)
    l1_words = filter(None, l1_words)
    l2_words = filter(None, l2_words)
    l1_words = [x.replace(u'\xa0', u'').lower().strip() for x in list(l1_words)]
    l2_words = [x.replace(u'\xa0', u'').lower().strip() for x in list(l2_words)]
    return l2_words, l1_words


def read_excel_matrix(filename):
    rb = load_workbook(filename=filename, read_only=True, data_only=True)
    rs = rb.worksheets[0]
    matrix = numpy.zeros((rs.max_row - 1, rs.max_column - 1))
    # -------------------------------
    language_cell = rs.cell(row=1, column=1).value
    language1_letters = {}
    language2_letters = {}
    # -------------------------------
    for index, rownum in enumerate(range(1, rs.max_row)):
        if index == 0:
            tmp = list(rs[rownum])[1:]
            for ind, letter in enumerate(tmp):
                language2_letters[letter.value] = ind
        else:
            row = list(rs[rownum])
            row = [x.value for x in row]
            language1_letters[row[0]] = index - 1
            matrix[index] = row[1:]
    return language_cell, language1_letters, language2_letters, matrix[1:]


def levenshtein(s1, s2, lang1_words, lang2_words, lang_cell, lang1, lang2, letter_matrix):
    flag = False
    if len(s1) < len(s2):
        return levenshtein(s2, s1, lang2_words, lang1_words, lang_cell, lang1, lang2, letter_matrix)
    if len(s2) == 0:
        return len(s1)
    if lang1_words.lower() == lang_cell.split('/')[0].lower() and lang2_words.lower() == lang_cell.split('/')[
        1].lower():
        pass
    elif lang1_words.lower() == lang_cell.split('/')[1].lower() and lang2_words.lower() == lang_cell.split('/')[
        0].lower():
        tmp = lang1
        lang1 = lang2
        lang2 = tmp
        flag = True
    else:
        print "The table provided has languages that don't coincide with word lists languages. Please provide appropriate table"
        return 0, 0, 0, 0, 0
    previous_row = []
    for i in range(len(s2) + 1):
        previous_row.append((i, i, 0, 0, ["D"] * i))
    for i, c1 in enumerate(s1):
        current_row = [(i + 1, 0, i + 1, 0, ["I"] * (i + 1))]
        for j, c2 in enumerate(s2):
            if flag == False:
                comp = float(letter_matrix[lang1[c1], lang2[c2]])
            else:
                comp = float(letter_matrix[lang2[c2], lang1[c1]])
                # hypothesize insertion
            total, deletions, insertions, substitutions, path = previous_row[j + 1]
            insert = (total + 1, deletions, insertions + 1, substitutions, path + ["I%s" % (c1)])
            # hypothesize deletion
            total, deletions, insertions, substitutions, path = current_row[j]
            delete = (total + 1, deletions + 1, insertions, substitutions, path + ["D%s" % (c2)])
            # hypothesize substitution
            total, deletions, insertions, substitutions, path = previous_row[j]
            substitute = (total + comp, deletions, insertions, substitutions + comp, path + ["S%s%s" % (c1, c2)])
            current_row.append(min(insert, delete, substitute))
        previous_row = current_row
    return previous_row[-1]


def print_line(list_s1, list_s2, lang_L1, lang_L2):
    slot_number = 0
    slots = []
    lang1_words = list_s1[0].encode("UTF-8")
    lang2_words = list_s2[0].encode("UTF-8")
    entropy_flag = False
    alignment_lang1 = []
    alignment_lang2 = []
    if lang_L1.lower() == lang1_words.lower() and lang_L2.lower() == lang2_words.lower():
        pass
    elif lang_L1.lower() == lang2_words.lower() and lang_L2.lower() == lang1_words.lower():
        entropy_flag = True
    else:
        print (lang_L1, type(lang_L1), lang_L2, type(lang_L2), lang1_words, type(lang1_words), lang2_words, type(
            lang2_words))
        sys.exit(
            "Languages you are trying to check do not coincide with the names of columns in the table. Please check")
    flag_lang = False
    # ---------------------------
    lev_dist = []
    entropy = {}
    for enum, word in enumerate(list_s1[1:]):
        s1 = word
        s2 = list_s2[enum + 1]
        if len(s1) < len(s2):
            tmp = s2
            s2 = s1
            s1 = tmp
            tmp_let = lang1_words
            lang1_words = lang2_words
            lang2_words = tmp_let
            flag_lang = True
        T, D, I, S, P = levenshtein(s1, s2, lang1_words, lang2_words, lang_cell, lang1, lang2, letter_matrix)
        if (T, D, I, S, P) != (0, 0, 0, 0, 0):
            cells1 = []
            cells2 = []
            i1 = 0
            i2 = 0
            for p in P:
                if p.startswith("D"):
                    cells1.append("")
                    cells2.append(s2[i2])
                    i2 += 1
                elif p.startswith("I"):
                    cells2.append("")
                    cells1.append(s1[i1])
                    i1 += 1
                elif p.startswith("S"):
                    cells1.append(s1[i1])
                    i1 += 1
                    cells2.append(s2[i2])
                    i2 += 1
            lev_dist.append(T)
            if (flag_lang == True and entropy_flag == False) or (flag_lang == False and entropy_flag == True):
                alignment_lang1.append(",".join(cells1))
                alignment_lang2.append(",".join(cells2))
            elif (flag_lang == False and entropy_flag == False) or (flag_lang == True and entropy_flag == True):
                alignment_lang1.append(",".join(cells2))
                alignment_lang2.append(",".join(cells1))
            entropy_pairs = []
            slot_number += len(P)
            slots.append((len(P)))
            cells1 = [(" ".decode("UTF-8") if x == "" else x) for x in cells1]
            cells2 = [(" ".decode("UTF-8") if x == "" else x) for x in cells2]
            for ind, el in enumerate(cells1):
                if (flag_lang == True and entropy_flag == False) or (flag_lang == False and entropy_flag == True):
                    entropy_pairs.append(cells2[ind] + ":" + el)
                    if (cells2[ind], el) in entropy:
                        entropy[cells2[ind], el] += 1
                    else:
                        entropy[cells2[ind], el] = 1
                elif (flag_lang == False and entropy_flag == False) or (flag_lang == True and entropy_flag == True):
                    entropy_pairs.append(el + ":" + cells2[ind])
                    if (el, cells2[ind]) in entropy:
                        entropy[el, cells2[ind]] += 1
                    else:
                        entropy[el, cells2[ind]] = 1
            if flag_lang == True:
                tmp_lang = lang1_words
                lang1_words = lang2_words
                lang2_words = tmp_lang
                flag_lang = False
    return entropy, slot_number, slots, alignment_lang1, alignment_lang2


def transformation_probability_letter(entropy_dict):
    transformation_probability_list = []
    for el in entropy_dict:
        tmp = 0
        for elem in entropy_dict:
            if elem[0] == el[0]:
                tmp += entropy_dict[elem]
        transformation_probability_list.append(el[0] + ", " + el[1] + ", " + str(float(int(entropy_dict[el]) / tmp)))
    return transformation_probability_list


# def full_entropy(conditional_entropy_dict, l2_probab_dict, slot_number):
#     entropy_value = 0
#     for el in conditional_entropy_dict:
#         entropy_value += conditional_entropy_dict[el] * l2_probab_dict[el] / slot_number
#     return entropy_value


if __name__ == '__main__':
    try:
        arguments = docopt.docopt(__doc__)
        words_file = arguments["<words_file>"]
        matrix_file = arguments["<matrix_file>"]
        file_to_write = arguments["<file_to_write>"]
        lang_from = arguments["<lang_from>"]
        lang_to = arguments["<lang_to>"]
    except docopt.DocoptExit as e:
        print (e.message)


    l1_words, l2_words = read_excel(words_file, 1, 0)
    lang_cell, lang1, lang2, letter_matrix = read_excel_matrix(matrix_file)

    entropy, slot_number, slots, alignment_lang1, alignment_lang2 = print_line(l1_words, l2_words, lang_from, lang_to)
    transf_prob = transformation_probability_letter(entropy)

    wb = Workbook()
    ws = wb.active
    ws.title = "result sheet"
    for el in transf_prob:
        tmp_el = el.split(", ")
        if log(float(tmp_el[2]), 2) != 0:
            ws.append([tmp_el[0], tmp_el[1], str(-1 * log(float(tmp_el[2]), 2))])
        else:
            ws.append([tmp_el[0], tmp_el[1], str(log(float(tmp_el[2]), 2))])
    wb.save(file_to_write)
